# frozen_string_literal: true

module SuperDiff
  module RSpec
    module InspectionTreeBuilders
      # TODO: Is this needed?
      class Primitive < Basic::InspectionTreeBuilders::Primitive
      end
    end
  end
end
