# frozen_string_literal: true

module SuperDiff
  module ActiveSupport
    module OperationTreeFlatteners
      autoload(
        :HashWithIndifferentAccess,
        'super_diff/active_support/operation_tree_flatteners/hash_with_indifferent_access'
      )
    end
  end
end
